#include "accessing.h"

/* Reserve this mapping and return whether 'fileid' is associated with an
   accessor. */

bool Accessing::reserve(fileid_t fileid)
{
    _lock.lock();

    return _accessors.find(fileid) != _accessors.end();
}

/* Return the accessor for 'fileid' and release this mapping. */

Accessor *Accessing::get(fileid_t fileid)
{
    FileMapping::iterator entry = _accessors.find(fileid);
    Accessor *accessor;

    if (entry == _accessors.end())
        accessor = NULL;
    else
        accessor = entry->second;

    _lock.unlock();
    return accessor;
}

/* Associate 'fileid' with the given 'accessor' and release this mapping. */

void Accessing::set(fileid_t fileid, Accessor *accessor)
{
    FileMapping::iterator entry = _accessors.find(fileid);

    if (entry == _accessors.end())
        _accessors[fileid] = accessor;

    _lock.unlock();
}

// vim: tabstop=4 expandtab shiftwidth=4
