#include "pages_conserving.h"

PagesConserving::PagesConserving(Memory *memory)
: PageCollection(memory)
{
    _queue = &_page_queue;
}

PagesConserving::PagesConserving()
: PageCollection()
{
    _queue = &_page_queue;
}



/* Refuse to obtain a new flexpage directly. All flexpages must be obtained from
   the page queue. */

Flexpage *PagesConserving::flexpage()
{
    return NULL;
}



/* Decrease the provision of flexpages to pipes. */

bool PagesConserving::decrease(offset_t size)
{
    // NOTE: Need to remove flexpages from the queue.
    // NOTE: This might need to happen incrementally in case some pages are not
    // NOTE: back in the queue for whatever reason.

    return true;
}

/* Increase the provision of flexpages to pipes, returning whether allocation
   succeeded. */

bool PagesConserving::increase(offset_t size)
{
    offset_t allocated_size = 0;
    std::list<Region *> allocated;
    std::list<Region *>::iterator it;

    /* Allocate regions for the defined increment value. */

    while (allocated_size < size)
    {
        Region *r = _memory->region();

        /* Free all allocated regions if not all regions can be allocated. */

        if (r == NULL)
        {
            for (it = allocated.begin(); it != allocated.end(); it++)
                _memory->release(r);

            return false;
        }

        allocated.push_back(r);
        allocated_size += r->size();
    }

    /* Queue flexpages for the regions. */

    for (it = allocated.begin(); it != allocated.end(); it++)
        _queue->push(NULL, new Flexpage(*it));

    return true;
}

// vim: tabstop=4 expandtab shiftwidth=4
